var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
import { DOCUMENT } from '@angular/common';
import { ChangeDetectionStrategy, Component, ElementRef, EventEmitter, HostListener, Inject, Input, NgZone, Optional, Output, Renderer2, TemplateRef, ViewChild, ViewEncapsulation } from '@angular/core';
import { animationFrameScheduler, asapScheduler, fromEvent, merge, Subject } from 'rxjs';
import { auditTime, takeUntil } from 'rxjs/operators';
import { SelectService } from './select.service';
import { isDefined } from './select.utils';
var TOP_CSS_CLASS = 'hi-select-top';
var BOTTOM_CSS_CLASS = 'hi-select-bottom';
var SCROLL_SCHEDULER = typeof requestAnimationFrame !== 'undefined' ? animationFrameScheduler : asapScheduler;
var SelectDropdown = (function () {
    function SelectDropdown(_renderer, _zone, _panelService, _elementRef, _document) {
        this._renderer = _renderer;
        this._zone = _zone;
        this._panelService = _panelService;
        this._document = _document;
        this.items = [];
        this.position = 'auto';
        this.virtualScroll = false;
        this.filterValue = null;
        this.update = new EventEmitter();
        this.scroll = new EventEmitter();
        this.scrollToEnd = new EventEmitter();
        this.outsideClick = new EventEmitter();
        this._destroy$ = new Subject();
        this._scrollToEndFired = false;
        this._updateScrollHeight = false;
        this._lastScrollPosition = 0;
        this._dropdown = _elementRef.nativeElement;
    }
    Object.defineProperty(SelectDropdown.prototype, "currentPosition", {
        get: function () {
            return this._currentPosition;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectDropdown.prototype, "itemsLength", {
        get: function () {
            return this._itemsLength;
        },
        set: function (value) {
            if (value !== this._itemsLength) {
                this._itemsLength = value;
                this._onItemsLengthChanged();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectDropdown.prototype, "_startOffset", {
        get: function () {
            if (this.markedItem) {
                var _a = this._panelService.dimensions, itemHeight = _a.itemHeight, panelHeight = _a.panelHeight;
                var offset = this.markedItem.index * itemHeight;
                return panelHeight > offset ? 0 : offset;
            }
            return 0;
        },
        enumerable: true,
        configurable: true
    });
    SelectDropdown.prototype.handleMousedown = function ($event) {
        var target = $event.target;
        if (target.tagName === 'INPUT') {
            return;
        }
        $event.preventDefault();
    };
    SelectDropdown.prototype.ngOnInit = function () {
        this._select = this._dropdown.parentElement;
        this._virtualPadding = this.paddingElementRef.nativeElement;
        this._scrollablePanel = this.scrollElementRef.nativeElement;
        this._contentPanel = this.contentElementRef.nativeElement;
        this._handleScroll();
        this._handleOutsideClick();
        this._appendDropdown();
    };
    SelectDropdown.prototype.ngOnChanges = function (changes) {
        if (changes.items) {
            var change = changes.items;
            this._onItemsChange(change.currentValue, change.firstChange);
        }
    };
    SelectDropdown.prototype.ngOnDestroy = function () {
        this._destroy$.next();
        this._destroy$.complete();
        this._destroy$.unsubscribe();
        if (this.appendTo) {
            this._renderer.removeChild(this._dropdown.parentNode, this._dropdown);
        }
    };
    SelectDropdown.prototype.scrollTo = function (option, startFromOption) {
        if (startFromOption === void 0) { startFromOption = false; }
        if (!option) {
            return;
        }
        var index = this.items.indexOf(option);
        if (index < 0 || index >= this.itemsLength) {
            return;
        }
        var scrollTo;
        if (this.virtualScroll) {
            var itemHeight = this._panelService.dimensions.itemHeight;
            scrollTo = this._panelService.getScrollTo(index * itemHeight, itemHeight, this._lastScrollPosition);
        }
        else {
            var item = this._dropdown.querySelector("#" + option.htmlId);
            var lastScroll = startFromOption ? item.offsetTop : this._lastScrollPosition;
            scrollTo = this._panelService.getScrollTo(item.offsetTop, item.clientHeight, lastScroll);
        }
        if (isDefined(scrollTo)) {
            this._scrollablePanel.scrollTop = scrollTo;
        }
    };
    SelectDropdown.prototype.scrollToTag = function () {
        var panel = this._scrollablePanel;
        panel.scrollTop = panel.scrollHeight - panel.clientHeight;
    };
    SelectDropdown.prototype.adjustPosition = function () {
        var parent = this._parent.getBoundingClientRect();
        var select = this._select.getBoundingClientRect();
        this._setOffset(parent, select);
    };
    SelectDropdown.prototype._handleDropdownPosition = function () {
        this._currentPosition = this._calculateCurrentPosition(this._dropdown);
        if (this._currentPosition === 'top') {
            this._renderer.addClass(this._dropdown, TOP_CSS_CLASS);
            this._renderer.removeClass(this._dropdown, BOTTOM_CSS_CLASS);
            this._renderer.addClass(this._select, TOP_CSS_CLASS);
            this._renderer.removeClass(this._select, BOTTOM_CSS_CLASS);
        }
        else {
            this._renderer.addClass(this._dropdown, BOTTOM_CSS_CLASS);
            this._renderer.removeClass(this._dropdown, TOP_CSS_CLASS);
            this._renderer.addClass(this._select, BOTTOM_CSS_CLASS);
            this._renderer.removeClass(this._select, TOP_CSS_CLASS);
        }
        if (this.appendTo) {
            this._updatePosition();
        }
        this._dropdown.style.opacity = '1';
    };
    SelectDropdown.prototype._handleScroll = function () {
        var _this = this;
        this._zone.runOutsideAngular(function () {
            fromEvent(_this.scrollElementRef.nativeElement, 'scroll')
                .pipe(takeUntil(_this._destroy$), auditTime(0, SCROLL_SCHEDULER))
                .subscribe(function (e) { return _this._onContentScrolled(e.target.scrollTop); });
        });
    };
    SelectDropdown.prototype._handleOutsideClick = function () {
        var _this = this;
        if (!this._document) {
            return;
        }
        this._zone.runOutsideAngular(function () {
            merge(fromEvent(_this._document, 'touchstart', { capture: true }), fromEvent(_this._document, 'mousedown', { capture: true })).pipe(takeUntil(_this._destroy$))
                .subscribe(function ($event) { return _this._checkToClose($event); });
        });
    };
    SelectDropdown.prototype._checkToClose = function ($event) {
        var _this = this;
        if (this._select.contains($event.target) || this._dropdown.contains($event.target)) {
            return;
        }
        var path = $event.path || ($event.composedPath && $event.composedPath());
        if ($event.target && $event.target.shadowRoot && path && path[0] && this._select.contains(path[0])) {
            return;
        }
        this._zone.run(function () { return _this.outsideClick.emit(); });
    };
    SelectDropdown.prototype._onItemsChange = function (items, firstChange) {
        this.items = items || [];
        this._scrollToEndFired = false;
        this.itemsLength = items.length;
        if (this.virtualScroll) {
            this._updateItemsRange(firstChange);
        }
        else {
            this._updateItems(firstChange);
        }
    };
    SelectDropdown.prototype._updateItems = function (firstChange) {
        var _this = this;
        this.update.emit(this.items);
        if (firstChange === false) {
            return;
        }
        this._zone.runOutsideAngular(function () {
            Promise.resolve().then(function () {
                var panelHeight = _this._scrollablePanel.clientHeight;
                _this._panelService.setDimensions(0, panelHeight);
                _this._handleDropdownPosition();
                _this.scrollTo(_this.markedItem, firstChange);
            });
        });
    };
    SelectDropdown.prototype._updateItemsRange = function (firstChange) {
        var _this = this;
        this._zone.runOutsideAngular(function () {
            _this._measureDimensions().then(function () {
                if (firstChange) {
                    _this._renderItemsRange(_this._startOffset);
                    _this._handleDropdownPosition();
                }
                else {
                    _this._renderItemsRange();
                }
            });
        });
    };
    SelectDropdown.prototype._onContentScrolled = function (scrollTop) {
        if (this.virtualScroll) {
            this._renderItemsRange(scrollTop);
        }
        this._lastScrollPosition = scrollTop;
        this._fireScrollToEnd(scrollTop);
    };
    SelectDropdown.prototype._updateVirtualHeight = function (height) {
        if (this._updateScrollHeight) {
            this._virtualPadding.style.height = height + "px";
            this._updateScrollHeight = false;
        }
    };
    SelectDropdown.prototype._onItemsLengthChanged = function () {
        this._updateScrollHeight = true;
    };
    SelectDropdown.prototype._renderItemsRange = function (scrollTop) {
        var _this = this;
        if (scrollTop === void 0) { scrollTop = null; }
        if (scrollTop && this._lastScrollPosition === scrollTop) {
            return;
        }
        scrollTop = scrollTop || this._scrollablePanel.scrollTop;
        var range = this._panelService.calculateItems(scrollTop, this.itemsLength, this.bufferAmount);
        this._updateVirtualHeight(range.scrollHeight);
        this._contentPanel.style.transform = "translateY(" + range.topPadding + "px)";
        this._zone.run(function () {
            _this.update.emit(_this.items.slice(range.start, range.end));
            _this.scroll.emit({ start: range.start, end: range.end });
        });
        if (isDefined(scrollTop) && this._lastScrollPosition === 0) {
            this._scrollablePanel.scrollTop = scrollTop;
            this._lastScrollPosition = scrollTop;
        }
    };
    SelectDropdown.prototype._measureDimensions = function () {
        var _this = this;
        if (this._panelService.dimensions.itemHeight > 0 || this.itemsLength === 0) {
            return Promise.resolve(this._panelService.dimensions);
        }
        var first = this.items[0];
        this.update.emit([first]);
        return Promise.resolve().then(function () {
            var option = _this._dropdown.querySelector("#" + first.htmlId);
            var optionHeight = option.clientHeight;
            _this._virtualPadding.style.height = optionHeight * _this.itemsLength + "px";
            var panelHeight = _this._scrollablePanel.clientHeight;
            _this._panelService.setDimensions(optionHeight, panelHeight);
            return _this._panelService.dimensions;
        });
    };
    SelectDropdown.prototype._fireScrollToEnd = function (scrollTop) {
        var _this = this;
        if (this._scrollToEndFired || scrollTop === 0) {
            return;
        }
        var padding = this.virtualScroll ?
            this._virtualPadding :
            this._contentPanel;
        if (scrollTop + this._dropdown.clientHeight >= padding.clientHeight) {
            this._zone.run(function () { return _this.scrollToEnd.emit(); });
            this._scrollToEndFired = true;
        }
    };
    SelectDropdown.prototype._calculateCurrentPosition = function (dropdownEl) {
        if (this.position !== 'auto') {
            return this.position;
        }
        var selectRect = this._select.getBoundingClientRect();
        var scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
        var offsetTop = selectRect.top + window.pageYOffset;
        var height = selectRect.height;
        var dropdownHeight = dropdownEl.getBoundingClientRect().height;
        if (offsetTop + height + dropdownHeight > scrollTop + document.documentElement.clientHeight) {
            return 'top';
        }
        else {
            return 'bottom';
        }
    };
    SelectDropdown.prototype._appendDropdown = function () {
        if (!this.appendTo) {
            return;
        }
        this._parent = document.querySelector(this.appendTo);
        if (!parent) {
            throw new Error("appendTo selector " + this.appendTo + " did not found any parent element");
        }
        this._parent.appendChild(this._dropdown);
    };
    SelectDropdown.prototype._updatePosition = function () {
        var select = this._select.getBoundingClientRect();
        var parent = this._parent.getBoundingClientRect();
        var offsetLeft = select.left - parent.left;
        this._setOffset(parent, select);
        this._dropdown.style.left = offsetLeft + 'px';
        this._dropdown.style.width = select.width + 'px';
        this._dropdown.style.minWidth = select.width + 'px';
    };
    SelectDropdown.prototype._setOffset = function (parent, select) {
        var delta = select.height;
        if (this._currentPosition === 'top') {
            var offsetBottom = parent.bottom - select.bottom;
            this._dropdown.style.bottom = offsetBottom + delta + 'px';
            this._dropdown.style.top = 'auto';
        }
        else if (this._currentPosition === 'bottom') {
            var offsetTop = select.top - parent.top;
            this._dropdown.style.top = offsetTop + delta + 'px';
            this._dropdown.style.bottom = 'auto';
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], SelectDropdown.prototype, "items", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectDropdown.prototype, "markedItem", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SelectDropdown.prototype, "position", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SelectDropdown.prototype, "appendTo", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectDropdown.prototype, "bufferAmount", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectDropdown.prototype, "virtualScroll", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], SelectDropdown.prototype, "headerTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], SelectDropdown.prototype, "footerTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SelectDropdown.prototype, "filterValue", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SelectDropdown.prototype, "update", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SelectDropdown.prototype, "scroll", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SelectDropdown.prototype, "scrollToEnd", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SelectDropdown.prototype, "outsideClick", void 0);
    __decorate([
        ViewChild('content', { read: ElementRef, static: true }),
        __metadata("design:type", ElementRef)
    ], SelectDropdown.prototype, "contentElementRef", void 0);
    __decorate([
        ViewChild('scroll', { read: ElementRef, static: true }),
        __metadata("design:type", ElementRef)
    ], SelectDropdown.prototype, "scrollElementRef", void 0);
    __decorate([
        ViewChild('padding', { read: ElementRef, static: true }),
        __metadata("design:type", ElementRef)
    ], SelectDropdown.prototype, "paddingElementRef", void 0);
    __decorate([
        HostListener('mousedown', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [MouseEvent]),
        __metadata("design:returntype", void 0)
    ], SelectDropdown.prototype, "handleMousedown", null);
    SelectDropdown = __decorate([
        Component({
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            selector: 'ng-dropdown-panel',
            template: "\n      <div *ngIf=\"headerTemplate\" class=\"ng-dropdown-header\">\n          <ng-container [ngTemplateOutlet]=\"headerTemplate\" [ngTemplateOutletContext]=\"{ searchTerm: filterValue }\"></ng-container>\n      </div>\n      <div #scroll class=\"ng-dropdown-panel-items scroll-host\">\n          <div #padding [class.total-padding]=\"virtualScroll\"></div>\n          <div #content [class.scrollable-content]=\"virtualScroll && items.length\">\n              <ng-content></ng-content>\n          </div>\n      </div>\n      <div *ngIf=\"footerTemplate\" class=\"ng-dropdown-footer\">\n          <ng-container [ngTemplateOutlet]=\"footerTemplate\" [ngTemplateOutletContext]=\"{ searchTerm: filterValue }\"></ng-container>\n      </div>\n  "
        }),
        __param(4, Optional()), __param(4, Inject(DOCUMENT)),
        __metadata("design:paramtypes", [Renderer2,
            NgZone,
            SelectService,
            ElementRef, Object])
    ], SelectDropdown);
    return SelectDropdown;
}());
export { SelectDropdown };
