var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { isDefined, isFunction, isObject, newId, stripChars } from './select.utils';
var SelectItems = (function () {
    function SelectItems(_ngSelect, _selectionModel) {
        this._ngSelect = _ngSelect;
        this._selectionModel = _selectionModel;
        this._items = [];
        this._filteredItems = [];
        this._markedIndex = -1;
    }
    Object.defineProperty(SelectItems.prototype, "items", {
        get: function () {
            return this._items;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectItems.prototype, "filteredItems", {
        get: function () {
            return this._filteredItems;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectItems.prototype, "markedIndex", {
        get: function () {
            return this._markedIndex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectItems.prototype, "selectedItems", {
        get: function () {
            return this._selectionModel.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectItems.prototype, "markedItem", {
        get: function () {
            return this._filteredItems[this._markedIndex];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectItems.prototype, "noItemsToSelect", {
        get: function () {
            return this._ngSelect.hideSelected && this._items.length === this.selectedItems.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectItems.prototype, "maxItemsSelected", {
        get: function () {
            return this._ngSelect.multiple && this._ngSelect.maxSelectedItems <= this.selectedItems.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectItems.prototype, "lastSelectedItem", {
        get: function () {
            var i = this.selectedItems.length - 1;
            for (; i >= 0; i--) {
                var item = this.selectedItems[i];
                if (!item.disabled) {
                    return item;
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    SelectItems.prototype.setItems = function (items) {
        var _this = this;
        this._items = items.map(function (item, index) { return _this.mapItem(item, index); });
        if (this._ngSelect.groupBy) {
            this._groups = this._groupBy(this._items, this._ngSelect.groupBy);
            this._items = this._flatten(this._groups);
        }
        else {
            this._groups = new Map();
            this._groups.set(undefined, this._items);
        }
        this._filteredItems = this._items.slice();
    };
    SelectItems.prototype.select = function (item) {
        if (item.selected || this.maxItemsSelected) {
            return;
        }
        var multiple = this._ngSelect.multiple;
        if (!multiple) {
            this.clearSelected();
        }
        this._selectionModel.select(item, multiple, this._ngSelect.selectableGroupAsModel);
        if (this._ngSelect.hideSelected) {
            this._hideSelected(item);
        }
    };
    SelectItems.prototype.unselect = function (item) {
        if (!item.selected) {
            return;
        }
        this._selectionModel.unselect(item, this._ngSelect.multiple);
        if (this._ngSelect.hideSelected && isDefined(item.index) && this._ngSelect.multiple) {
            this._showSelected(item);
        }
    };
    SelectItems.prototype.findItem = function (value) {
        var _this = this;
        var findBy;
        if (this._ngSelect.compareWith) {
            findBy = function (item) { return _this._ngSelect.compareWith(item.value, value); };
        }
        else if (this._ngSelect.bindValue) {
            findBy = function (item) { return !item.children && _this.resolveNested(item.value, _this._ngSelect.bindValue) === value; };
        }
        else {
            findBy = function (item) { return item.value === value ||
                !item.children && item.label && item.label === _this.resolveNested(value, _this._ngSelect.bindLabel); };
        }
        return this._items.find(function (item) { return findBy(item); });
    };
    SelectItems.prototype.addItem = function (item) {
        var option = this.mapItem(item, this._items.length);
        this._items.push(option);
        this._filteredItems.push(option);
        return option;
    };
    SelectItems.prototype.clearSelected = function (keepDisabled) {
        if (keepDisabled === void 0) { keepDisabled = false; }
        this._selectionModel.clear(keepDisabled);
        this._items.forEach(function (item) {
            item.selected = keepDisabled && item.selected && item.disabled;
            item.marked = false;
        });
        if (this._ngSelect.hideSelected) {
            this.resetFilteredItems();
        }
    };
    SelectItems.prototype.findByLabel = function (term) {
        term = stripChars(term).toLocaleLowerCase();
        return this.filteredItems.find(function (item) {
            var label = stripChars(item.label).toLocaleLowerCase();
            return label.substr(0, term.length) === term;
        });
    };
    SelectItems.prototype.filter = function (term) {
        if (!term) {
            this.resetFilteredItems();
            return;
        }
        this._filteredItems = [];
        term = this._ngSelect.searchFn ? term : stripChars(term).toLocaleLowerCase();
        var match = this._ngSelect.searchFn || this._defaultSearchFn;
        var hideSelected = this._ngSelect.hideSelected;
        var _loop_1 = function (key) {
            var _a;
            var matchedItems = [];
            for (var _i = 0, _b = this_1._groups.get(key); _i < _b.length; _i++) {
                var item = _b[_i];
                if (hideSelected && (item.parent && item.parent.selected || item.selected)) {
                    continue;
                }
                var searchItem = this_1._ngSelect.searchFn ? item.value : item;
                if (match(term, searchItem)) {
                    matchedItems.push(item);
                }
            }
            if (matchedItems.length > 0) {
                var last_1 = matchedItems.slice(-1)[0];
                if (last_1.parent) {
                    var head = this_1._items.find(function (x) { return x === last_1.parent; });
                    this_1._filteredItems.push(head);
                }
                (_a = this_1._filteredItems).push.apply(_a, matchedItems);
            }
        };
        var this_1 = this;
        for (var _i = 0, _a = Array.from(this._groups.keys()); _i < _a.length; _i++) {
            var key = _a[_i];
            _loop_1(key);
        }
    };
    SelectItems.prototype.resetFilteredItems = function () {
        if (this._filteredItems.length === this._items.length) {
            return;
        }
        if (this._ngSelect.hideSelected && this.selectedItems.length > 0) {
            this._filteredItems = this._items.filter(function (x) { return !x.selected; });
        }
        else {
            this._filteredItems = this._items;
        }
    };
    SelectItems.prototype.unmarkItem = function () {
        this._markedIndex = -1;
    };
    SelectItems.prototype.markNextItem = function () {
        this._stepToItem(+1);
    };
    SelectItems.prototype.markPreviousItem = function () {
        this._stepToItem(-1);
    };
    SelectItems.prototype.markItem = function (item) {
        this._markedIndex = this._filteredItems.indexOf(item);
    };
    SelectItems.prototype.markSelectedOrDefault = function (markDefault) {
        if (this._filteredItems.length === 0) {
            return;
        }
        var lastMarkedIndex = this._getLastMarkedIndex();
        if (lastMarkedIndex > -1) {
            this._markedIndex = lastMarkedIndex;
        }
        else {
            this._markedIndex = markDefault ? this.filteredItems.findIndex(function (x) { return !x.disabled; }) : -1;
        }
    };
    SelectItems.prototype.resolveNested = function (option, key) {
        if (!isObject(option)) {
            return option;
        }
        if (key.indexOf('.') === -1) {
            return option[key];
        }
        else {
            var keys = key.split('.');
            var value = option;
            for (var i = 0, len = keys.length; i < len; ++i) {
                if (value == null) {
                    return null;
                }
                value = value[keys[i]];
            }
            return value;
        }
    };
    SelectItems.prototype.mapItem = function (item, index) {
        var label = isDefined(item.$ngOptionLabel) ? item.$ngOptionLabel : this.resolveNested(item, this._ngSelect.bindLabel);
        var value = isDefined(item.$ngOptionValue) ? item.$ngOptionValue : item;
        return {
            index: index,
            label: isDefined(label) ? label.toString() : '',
            value: value,
            disabled: item.disabled,
            htmlId: this._ngSelect.dropdownId + "-" + index,
        };
    };
    SelectItems.prototype.mapSelectedItems = function () {
        var _this = this;
        var multiple = this._ngSelect.multiple;
        for (var _i = 0, _a = this.selectedItems; _i < _a.length; _i++) {
            var selected = _a[_i];
            var value = this._ngSelect.bindValue ? this.resolveNested(selected.value, this._ngSelect.bindValue) : selected.value;
            var item = isDefined(value) ? this.findItem(value) : null;
            this._selectionModel.unselect(selected, multiple);
            this._selectionModel.select(item || selected, multiple, this._ngSelect.selectableGroupAsModel);
        }
        if (this._ngSelect.hideSelected) {
            this._filteredItems = this.filteredItems.filter(function (x) { return _this.selectedItems.indexOf(x) === -1; });
        }
    };
    SelectItems.prototype._showSelected = function (item) {
        this._filteredItems.push(item);
        if (item.parent) {
            var parent_1 = item.parent;
            var parentExists = this._filteredItems.find(function (x) { return x === parent_1; });
            if (!parentExists) {
                this._filteredItems.push(parent_1);
            }
        }
        else if (item.children) {
            for (var _i = 0, _a = item.children; _i < _a.length; _i++) {
                var child = _a[_i];
                child.selected = false;
                this._filteredItems.push(child);
            }
        }
        this._filteredItems = this._filteredItems.sort(function (a, b) { return (a.index - b.index); }).slice();
    };
    SelectItems.prototype._hideSelected = function (item) {
        this._filteredItems = this._filteredItems.filter(function (x) { return x !== item; });
        if (item.parent) {
            var children = item.parent.children;
            if (children.every(function (x) { return x.selected; })) {
                this._filteredItems = this._filteredItems.filter(function (x) { return x !== item.parent; });
            }
        }
        else if (item.children) {
            this._filteredItems = this.filteredItems.filter(function (x) { return x.parent !== item; });
        }
    };
    SelectItems.prototype._defaultSearchFn = function (search, opt) {
        var label = stripChars(opt.label).toLocaleLowerCase();
        return label.indexOf(search) > -1;
    };
    SelectItems.prototype._getNextItemIndex = function (steps) {
        if (steps > 0) {
            return (this._markedIndex === this._filteredItems.length - 1) ? 0 : (this._markedIndex + 1);
        }
        return (this._markedIndex <= 0) ? (this._filteredItems.length - 1) : (this._markedIndex - 1);
    };
    SelectItems.prototype._stepToItem = function (steps) {
        if (this._filteredItems.length === 0 || this._filteredItems.every(function (x) { return x.disabled; })) {
            return;
        }
        this._markedIndex = this._getNextItemIndex(steps);
        if (this.markedItem.disabled) {
            this._stepToItem(steps);
        }
    };
    SelectItems.prototype._getLastMarkedIndex = function () {
        if (this._ngSelect.hideSelected) {
            return -1;
        }
        if (this._markedIndex > -1 && this.markedItem === undefined) {
            return -1;
        }
        var selectedIndex = this._filteredItems.indexOf(this.lastSelectedItem);
        if (this.lastSelectedItem && selectedIndex < 0) {
            return -1;
        }
        return Math.max(this.markedIndex, selectedIndex);
    };
    SelectItems.prototype._groupBy = function (items, prop) {
        var _this = this;
        var groups = new Map();
        if (items.length === 0) {
            return groups;
        }
        if (Array.isArray(items[0].value[prop])) {
            for (var _i = 0, items_1 = items; _i < items_1.length; _i++) {
                var item = items_1[_i];
                var children = (item.value[prop] || []).map(function (x, index) { return _this.mapItem(x, index); });
                groups.set(item, children);
            }
            return groups;
        }
        var isFnKey = isFunction(this._ngSelect.groupBy);
        var keyFn = function (item) {
            var key = isFnKey ? prop(item.value) : item.value[prop];
            return isDefined(key) ? key : undefined;
        };
        for (var _a = 0, items_2 = items; _a < items_2.length; _a++) {
            var item = items_2[_a];
            var key = keyFn(item);
            var group = groups.get(key);
            if (group) {
                group.push(item);
            }
            else {
                groups.set(key, [item]);
            }
        }
        return groups;
    };
    SelectItems.prototype._flatten = function (groups) {
        var isGroupByFn = isFunction(this._ngSelect.groupBy);
        var items = [];
        var _loop_2 = function (key) {
            var i = items.length;
            if (key === undefined) {
                var withoutGroup = groups.get(undefined) || [];
                items.push.apply(items, withoutGroup.map(function (x) { return (__assign({}, x, { index: i++ })); }));
                return "continue";
            }
            var isObjectKey = isObject(key);
            var parent_2 = {
                label: isObjectKey ? '' : String(key),
                children: undefined,
                parent: null,
                index: i++,
                disabled: !this_2._ngSelect.selectableGroup,
                htmlId: newId(),
            };
            var groupKey = isGroupByFn ? this_2._ngSelect.bindLabel : this_2._ngSelect.groupBy;
            var groupValue = this_2._ngSelect.groupValue || (function () {
                var _a;
                if (isObjectKey) {
                    return key.value;
                }
                return _a = {}, _a[groupKey] = key, _a;
            });
            var children = groups.get(key).map(function (x) {
                x.parent = parent_2;
                x.children = undefined;
                x.index = i++;
                return x;
            });
            parent_2.children = children;
            parent_2.value = groupValue(key, children.map(function (x) { return x.value; }));
            items.push(parent_2);
            items.push.apply(items, children);
        };
        var this_2 = this;
        for (var _i = 0, _a = Array.from(groups.keys()); _i < _a.length; _i++) {
            var key = _a[_i];
            _loop_2(key);
        }
        return items;
    };
    return SelectItems;
}());
export { SelectItems };
