export var KeyCode;
(function (KeyCode) {
    KeyCode[KeyCode["Tab"] = 9] = "Tab";
    KeyCode[KeyCode["Enter"] = 13] = "Enter";
    KeyCode[KeyCode["Esc"] = 27] = "Esc";
    KeyCode[KeyCode["Space"] = 32] = "Space";
    KeyCode[KeyCode["ArrowUp"] = 38] = "ArrowUp";
    KeyCode[KeyCode["ArrowDown"] = 40] = "ArrowDown";
    KeyCode[KeyCode["Backspace"] = 8] = "Backspace";
})(KeyCode || (KeyCode = {}));
var SelectService = (function () {
    function SelectService() {
        this._dimensions = {
            itemHeight: 0,
            panelHeight: 0,
            itemsPerViewport: 0
        };
    }
    Object.defineProperty(SelectService.prototype, "dimensions", {
        get: function () {
            return this._dimensions;
        },
        enumerable: true,
        configurable: true
    });
    SelectService.prototype.calculateItems = function (scrollPos, itemsLength, buffer) {
        var d = this._dimensions;
        var scrollHeight = d.itemHeight * itemsLength;
        var scrollTop = Math.max(0, scrollPos);
        var indexByScrollTop = scrollTop / scrollHeight * itemsLength;
        var end = Math.min(itemsLength, Math.ceil(indexByScrollTop) + (d.itemsPerViewport + 1));
        var maxStartEnd = end;
        var maxStart = Math.max(0, maxStartEnd - d.itemsPerViewport);
        var start = Math.min(maxStart, Math.floor(indexByScrollTop));
        var topPadding = d.itemHeight * Math.ceil(start) - (d.itemHeight * Math.min(start, buffer));
        topPadding = !isNaN(topPadding) ? topPadding : 0;
        start = !isNaN(start) ? start : -1;
        end = !isNaN(end) ? end : -1;
        start -= buffer;
        start = Math.max(0, start);
        end += buffer;
        end = Math.min(itemsLength, end);
        return {
            topPadding: topPadding,
            scrollHeight: scrollHeight,
            start: start,
            end: end
        };
    };
    SelectService.prototype.setDimensions = function (itemHeight, panelHeight) {
        var itemsPerViewport = Math.max(1, Math.floor(panelHeight / itemHeight));
        this._dimensions = {
            itemHeight: itemHeight,
            panelHeight: panelHeight,
            itemsPerViewport: itemsPerViewport
        };
    };
    SelectService.prototype.getScrollTo = function (itemTop, itemHeight, lastScroll) {
        var panelHeight = this.dimensions.panelHeight;
        var itemBottom = itemTop + itemHeight;
        var top = lastScroll;
        var bottom = top + panelHeight;
        if (panelHeight >= itemBottom && lastScroll === itemTop) {
            return null;
        }
        if (itemBottom > bottom) {
            return top + itemBottom - bottom;
        }
        else if (itemTop <= top) {
            return itemTop;
        }
        return null;
    };
    return SelectService;
}());
export { SelectService };
